



if not exist %PREFIX%\\Library\\include\\ignition\\sensors5\\ignition\\sensors.hh exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
if not exist %PREFIX%\\Library\\lib\\ignition-sensors5.lib exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
if not exist %PREFIX%\\Library\\bin\\ignition-sensors5.dll exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
if not exist %PREFIX%\\Library\\lib\\cmake\\ignition-sensors5\\ignition-sensors5-config.cmake exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
exit /B 0
